package com.massivecraft.massivecore.entity;

import com.massivecraft.massivecore.MassiveCore;
import com.massivecraft.massivecore.command.type.TypeUniverse;
import com.massivecraft.massivecore.store.Entity;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeSet;

public class Multiverse extends Entity<Multiverse> {
    // -------------------------------------------- //
    // META
    // -------------------------------------------- //

    public static Multiverse get(Object oid) {
        return MultiverseColl.get().get(oid);
    }

    // -------------------------------------------- //
    // FIELDS
    // -------------------------------------------- //

    protected Map<String, Set<String>> uw = new HashMap<>();


    // -------------------------------------------- //
    // CONSTRUCT
    // -------------------------------------------- //

    public Multiverse() {

    }

    // -------------------------------------------- //
    // UNIVERSE
    // -------------------------------------------- //

    public boolean containsUniverse(String universe) {
        return this.getUniverses().contains(universe);
    }

    public Set<String> newUniverse(String universe) {
        if (universe.equals(MassiveCore.DEFAULT)) {
            return null;
        }
        Set<String> ret = this.uw.get(universe);
        if (ret == null) {
            ret = new HashSet<>();
            this.uw.put(universe, ret);
            this.changed();
        }
        return ret;
    }

    public Set<String> delUniverse(String universe) {
        this.changed();
        return this.uw.remove(universe);
    }

    public Set<String> getUniverses() {
        Set<String> ret = new TreeSet<>(this.uw.keySet());
        ret.add(MassiveCore.DEFAULT);
        return ret;
    }

    public String getUniverseForWorldName(String worldName) {
        for (Entry<String, Set<String>> entry : this.uw.entrySet()) {
            String universe = entry.getKey();
            Set<String> worlds = entry.getValue();
            if (worlds.contains(worldName)) {
                return universe;
            }
        }
        return MassiveCore.DEFAULT;
    }

    // -------------------------------------------- //
    // UNIVERSE AND WORLD
    // -------------------------------------------- //

    public boolean clearUniverse(String universe) {
        Set<String> worlds = this.uw.get(universe);
        if (worlds == null) {
            return false;
        }
        worlds.clear();
        this.changed();
        return true;
    }

    public boolean setWorldUniverse(String worldName, String universe) {
        if (this.getUniverseForWorldName(worldName).equals(universe)) {
            return false;
        }
        this.removeWorld(worldName);
        if (!universe.equals(MassiveCore.DEFAULT)) {
            this.newUniverse(universe).add(worldName);
        }
        this.changed();
        return true;
    }

    // -------------------------------------------- //
    // WORLD
    // -------------------------------------------- //

    public boolean containsWorld(String worldName) {
        return this.getWorlds().contains(worldName);
    }

    public Set<String> getWorlds() {
        Set<String> ret = new TreeSet<>();
        for (Set<String> uworlds : this.uw.values()) {
            ret.addAll(uworlds);
        }
        return ret;
    }

    public Set<String> getWorlds(String universe) {
        Set<String> orig = this.uw.get(universe);
        if (orig == null) {
            return null;
        }

        Set<String> ret = new TreeSet<>(orig);

        return ret;
    }

    public boolean removeWorld(String worldName) {
        for (Set<String> worldNames : this.uw.values()) {
            if (worldNames.remove(worldName)) {
                this.changed();
                return true;
            }
        }
        return false;
    }

    // -------------------------------------------- //
    // ARG READERS
    // -------------------------------------------- //

    public TypeUniverse typeUniverse() {
        return new TypeUniverse(this);
    }

}
