package com.massivecraft.massivecore.store;

import com.google.gson.Gson;
import com.massivecraft.massivecore.Identified;
import com.massivecraft.massivecore.MassiveCore;
import com.massivecraft.massivecore.store.accessor.Accessor;
import com.massivecraft.massivecore.util.MUtil;

import java.lang.ref.WeakReference;
import java.util.Objects;

public class EntityInternal<E extends EntityInternal<E>> implements Identified {
    // -------------------------------------------- //
    // FIELDS
    // -------------------------------------------- //

    protected transient WeakReference<EntityContainer<E>> container = new WeakReference<>(null);

    protected void setContainer(EntityContainer<E> container) {
        this.container = new WeakReference<>(container);
    }

    public EntityContainer<E> getContainer() {
        return this.container.get();
    }

    public Coll<?> getColl() {
        return this.getContainer().getColl();
    }

    protected transient String id;

    protected void setId(String id) {
        this.id = id;
    }

    public String getId() {
        return this.id;
    }

    public String getIdOrThrow() {
        String id = this.getId();
        if (id == null) {
            throw new NullPointerException("id");
        }
        return id;
    }

    // -------------------------------------------- //
    // ATTACH AND DETACH
    // -------------------------------------------- //

    public boolean attached() {
        return this.getContainer() != null && this.getId() != null;
    }

    public boolean detached() {
        return !this.attached();
    }

    public void preAttach(String id) {

    }

    public void postAttach(String id) {

    }

    public void preDetach(String id) {

    }

    public void postDetach(String id) {

    }

    @SuppressWarnings("unchecked")
    public E detach() {
        EntityContainer<E> coll = this.getContainer();
        if (coll == null) {
            return (E) this;
        }

        return coll.detachEntity((E) this);
    }

    // -------------------------------------------- //
    // SYNC AND IO ACTIONS
    // -------------------------------------------- //

    public boolean isLive() {
        String id = this.getId();
        if (id == null) {
            return false;
        }

        EntityContainer<E> container = this.getContainer();
        if (container == null) {
            return false;
        }

        if (!container.isLive()) {
            return false;
        }

        return true;
    }

    public void changed() {
        if (!this.isLive()) {
            return;
        }

        //System.out.println(this.getColl().getName() + ": " +this.getId() + " was modified locally");

        this.getContainer().putIdentifiedModificationFixed(this.getId(), Modification.UNKNOWN_CHANGED);
    }

    // -------------------------------------------- //
    // DERPINGTON
    // -------------------------------------------- //

    @SuppressWarnings("unchecked")
    public E load(E that) {
        Objects.requireNonNull(that, "that");
        Accessor.get(this.getClass()).copy(that, this);
        return (E) this;
    }

    public boolean isDefault() {
        return false;
    }

    // -------------------------------------------- //
    // CONVENIENCE: DATABASE
    // -------------------------------------------- //

    // GENERIC
    public <T> T convertGet(T value, T standard) {
        return value != null ? value : standard;
    }

    public <T> T convertSet(T value, T standard) {
        this.changed();
        return Objects.equals(value, standard) ? null : value;
    }

    public <T> T convertSet(T value, T current, T standard) {
        current = this.convertGet(current, standard);
        this.changed(value, current);
        return Objects.equals(value, standard) ? null : value;
    }

    public void changed(Object o1, Object o2) {
        if (MUtil.equalsishObject(o1, o2)) {
            return;
        }
        changed();
    }

    // BOOLEAN
    public boolean convertGet(Boolean value) {
        return convertGet(value, false);
    }

    public Boolean convertSet(Boolean value) {
        return convertSet(value, false);
    }

    // -------------------------------------------- //
    // STANDARDS
    // -------------------------------------------- //

    @Override
    public String toString() {
        Gson gson = MassiveCore.gson;
        Coll<?> coll = this.getColl();
        if (coll != null) {
            gson = coll.getGson();
        }

        return this.getClass().getSimpleName() + gson.toJson(this, this.getClass());
    }

}
